
CREATE PROC spSetupPerfTrack
AS
BEGIN
IF EXISTS (SELECT name FROM sys.objects WHERE name='PerfTrack')
BEGIN
DROP TABLE PerfTrack
END
CREATE TABLE [dbo].[PerfTrack](
	ID int identity(1,1),
	[QueryHash] [binary](8) NOT NULL,
	[CreationTime] [datetime] NOT NULL,
	[ExecutionCount] [int] NOT NULL,
	[TotalWorkerTime] [int] NOT NULL,
	[TotalPhysicalReads] [int] NOT NULL,
	[TotalLogicalReads] [int] NOT NULL,
	[TotalLogicalWrites] [int] NOT NULL,
	[TotalRows] [int] NOT NULL,
	[StatementText] [varchar](max) NULL,
	[QueryPlan] [xml] NULL,
	CONSTRAINT [PK_PerfTrack] PRIMARY KEY CLUSTERED 
	(
	[ID] ASC
	)
)
END;
GO

CREATE PROC spStartPerfTrack
AS
BEGIN
TRUNCATE TABLE PerfTrack
-- Find top queries
INSERT INTO PerfTrack
SELECT 
	query_stats.query_hash,
	query_stats.creation_time,
	query_stats.execution_count,
    query_stats.total_worker_time,
    query_stats.total_physical_reads,
    query_stats.total_logical_reads,
    query_stats.total_logical_writes,
    query_stats.total_rows,
    query_stats.statement_text,
	query_stats.query_plan
FROM
(SELECT 
	QS.*, 
	SUBSTRING(ST.text, (QS.statement_start_offset/2) + 1, ((CASE statement_end_offset WHEN -1 THEN DATALENGTH(st.text) ELSE QS.statement_end_offset END - QS.statement_start_offset)/2) + 1) AS statement_text,
    QP.query_plan
FROM sys.dm_exec_query_stats AS QS
INNER JOIN 
(SELECT 
	query_hash,
	MAX(creation_time) AS creation_time
FROM sys.dm_exec_query_stats AS QS
	GROUP BY query_hash ) as UQ ON QS.query_hash=UQ.query_hash AND QS.creation_time=UQ.creation_time
CROSS APPLY sys.dm_exec_sql_text(QS.sql_handle) as ST
CROSS APPLY sys.dm_exec_query_plan(QS.plan_handle) as QP) as query_stats

ORDER BY 3 DESC
END;
GO

CREATE PROC spStopPerfTrack
AS
BEGIN
SELECT
	query_stats.query_hash,
	query_stats.creation_time,
    query_stats.execution_count - COALESCE(pf.ExecutionCount,0) AS execution_count,
    query_stats.total_worker_time - COALESCE(pf.TotalWorkerTime,0) AS total_worker_time,
    total_worker_time / execution_count as avg_worker_time,
    query_stats.total_physical_reads - COALESCE(pf.TotalPhysicalReads,0) AS total_physical_reads,
    query_stats.total_logical_reads - COALESCE(pf.TotalLogicalReads,0) AS total_logical_reads,
    query_stats.total_logical_writes - COALESCE(TotalLogicalWrites,0) AS total_logical_writes,
    query_stats.total_rows - COALESCE(TotalRows,0) AS total_rows,
    query_stats.statement_text,
	query_stats.query_plan
FROM
(SELECT 
	QS.*, 
	SUBSTRING(ST.text, (QS.statement_start_offset/2) + 1, ((CASE statement_end_offset WHEN -1 THEN DATALENGTH(st.text) ELSE QS.statement_end_offset END - QS.statement_start_offset)/2) + 1) AS statement_text,
    QP.query_plan
FROM sys.dm_exec_query_stats AS QS
INNER JOIN 
(SELECT 
	query_hash,
	MAX(creation_time) AS creation_time
FROM sys.dm_exec_query_stats AS QS
	GROUP BY query_hash ) as UQ ON QS.query_hash=UQ.query_hash AND QS.creation_time=UQ.creation_time
CROSS APPLY sys.dm_exec_sql_text(QS.sql_handle) as ST
CROSS APPLY sys.dm_exec_query_plan(QS.plan_handle) as QP) as query_stats
LEFT JOIN PerfTrack pf on  query_stats.query_hash = pf.QueryHash
WHERE query_stats.execution_count - COALESCE(pf.ExecutionCount,0) >0
AND statement_text NOT LIKE 'INSERT INTO PerfTrack%'
ORDER BY 6 DESC
END;

-- Usage

-- Execute first on the database you want to monitor
exec spSetupPerfTrack

-- Execute every time you want to take the initial snapshot
exec spStartPerfTrack

-- Run your workload here

-- Execute to get the actual perf numbers
exec spStopPerfTrack

